from pathlib import Path

DEFAULT_CFG_HOME = Path.home() / '.config' / 'oven'
CFG_TEMP_URL = 'https://raw.githubusercontent.com/IsshikiHugh/ExpOven/refs/heads/main/docs/cfg.yaml.temp'
OVEN_VERSION_URL = 'https://raw.githubusercontent.com/IsshikiHugh/ExpOven/refs/heads/main/oven/version.py'

REQ_TIMEOUT = 60 * 3  # in seconds
